@@ -1,43 +1,19 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001,2002,2003, 2006 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 
 
@@ -49,6 +25,7 @@
 #include "libguile/print.h"
 #include "libguile/root.h"
 #include "libguile/smob.h"
+#include "libguile/deprecation.h"
 
 #include "libguile/validate.h"
 #include "libguile/macros.h"
@@ -61,8 +38,8 @@ macro_print (SCM macro, SCM port, scm_print_state *pstate)
 {
   SCM code = SCM_MACRO_CODE (macro);
   if (!SCM_CLOSUREP (code)
-      || SCM_FALSEP (scm_procedure_p (SCM_PRINT_CLOSURE))
-      || SCM_FALSEP (scm_printer_apply (SCM_PRINT_CLOSURE,
+      || scm_is_false (scm_procedure_p (SCM_PRINT_CLOSURE))
+      || scm_is_false (scm_printer_apply (SCM_PRINT_CLOSURE,
 					macro, port, pstate)))
     {
       if (!SCM_CLOSUREP (code))
@@ -72,10 +49,15 @@ macro_print (SCM macro, SCM port, scm_print_state *pstate)
 
       if (SCM_MACRO_TYPE (macro) == 0)
 	scm_puts ("syntax", port);
-      else if (SCM_MACRO_TYPE (macro) == 1)
+#if SCM_ENABLE_DEPRECATED == 1
+      if (SCM_MACRO_TYPE (macro) == 1)
 	scm_puts ("macro", port);
+#endif
       if (SCM_MACRO_TYPE (macro) == 2)
 	scm_puts ("macro!", port);
+      if (SCM_MACRO_TYPE (macro) == 3)
+	scm_puts ("builtin-macro!", port);
+
       scm_putc (' ', port);
       scm_iprin1 (scm_macro_name (macro), port, pstate);
 
@@ -84,7 +66,7 @@ macro_print (SCM macro, SCM port, scm_print_state *pstate)
 	  SCM formals = SCM_CLOSURE_FORMALS (code);
 	  SCM env = SCM_ENV (code);
 	  SCM xenv = SCM_EXTEND_ENV (formals, SCM_EOL, env);
-	  SCM src = scm_unmemocopy (SCM_CODE (code), xenv);
+	  SCM src = scm_i_unmemocopy_body (SCM_CODE (code), xenv);
 	  scm_putc (' ', port);
 	  scm_iprin1 (src, port, pstate);
 	}
@@ -95,38 +77,22 @@ macro_print (SCM macro, SCM port, scm_print_state *pstate)
   return 1;
 }
 
-
-SCM_DEFINE (scm_makacro, "procedure->syntax", 1, 0, 0,
-            (SCM code),
-	    "Return a @dfn{macro} which, when a symbol defined to this value\n"
-	    "appears as the first symbol in an expression, returns the\n"
-	    "result of applying @var{code} to the expression and the\n"
-	    "environment.")
-#define FUNC_NAME s_scm_makacro
+static SCM
+makmac (SCM code, scm_t_bits flags)
 {
-  SCM_VALIDATE_PROC (1,code);
-  SCM_RETURN_NEWSMOB (scm_tc16_macro, SCM_UNPACK (code));
+  SCM z;
+  SCM_NEWSMOB (z, scm_tc16_macro, SCM_UNPACK (code));
+  SCM_SET_SMOB_FLAGS (z, flags);
+  return z;
 }
-#undef FUNC_NAME
 
-
-SCM_DEFINE (scm_makmacro, "procedure->macro", 1, 0, 0, 
-           (SCM code),
-	    "Return a @dfn{macro} which, when a symbol defined to this value\n"
-	    "appears as the first symbol in an expression, evaluates the\n"
-	    "result of applying @var{code} to the expression and the\n"
-	    "environment.  For example:\n"
-	    "\n"
-	    "@lisp\n"
-	    "(define trace\n"
-	    "  (procedure->macro\n"
-	    "   (lambda (x env) `(set! ,(cadr x) (tracef ,(cadr x) ',(cadr x))))))\n\n"
-	    "(trace @i{foo}) @equiv{} (set! @i{foo} (tracef @i{foo} '@i{foo})).\n"
-	    "@end lisp")
-#define FUNC_NAME s_scm_makmacro
+/* Return a mmacro that is known to be one of guile's built in macros. */
+SCM
+scm_i_makbimacro (SCM code)
+#define FUNC_NAME "scm_i_makbimacro"
 {
-  SCM_VALIDATE_PROC (1,code);
-  SCM_RETURN_NEWSMOB (scm_tc16_macro | (1L << 16), SCM_UNPACK (code));
+  SCM_VALIDATE_PROC (1, code);
+  return makmac (code, 3);
 }
 #undef FUNC_NAME
 
@@ -143,26 +109,73 @@ SCM_DEFINE (scm_makmmacro, "procedure->memoizing-macro", 1, 0, 0,
 	    "form of the containing code.")
 #define FUNC_NAME s_scm_makmmacro
 {
-  SCM_VALIDATE_PROC (1,code);
-  SCM_RETURN_NEWSMOB (scm_tc16_macro | (2L << 16), SCM_UNPACK (code));
+  SCM_VALIDATE_PROC (1, code);
+  return makmac (code, 2);
 }
 #undef FUNC_NAME
 
 
+SCM_DEFINE (scm_makacro, "procedure->syntax", 1, 0, 0,
+            (SCM code),
+	    "Return a @dfn{macro} which, when a symbol defined to this value\n"
+	    "appears as the first symbol in an expression, returns the\n"
+	    "result of applying @var{code} to the expression and the\n"
+	    "environment.")
+#define FUNC_NAME s_scm_makacro
+{
+  SCM_VALIDATE_PROC (1, code);
+  return makmac (code, 0);
+}
+#undef FUNC_NAME
+
+
+#if SCM_ENABLE_DEPRECATED == 1
+
+SCM_DEFINE (scm_makmacro, "procedure->macro", 1, 0, 0, 
+           (SCM code),
+	    "Return a @dfn{macro} which, when a symbol defined to this value\n"
+	    "appears as the first symbol in an expression, evaluates the\n"
+	    "result of applying @var{code} to the expression and the\n"
+	    "environment.  For example:\n"
+	    "\n"
+	    "@lisp\n"
+	    "(define trace\n"
+	    "  (procedure->macro\n"
+	    "   (lambda (x env) `(set! ,(cadr x) (tracef ,(cadr x) ',(cadr x))))))\n\n"
+	    "(trace @i{foo}) @equiv{} (set! @i{foo} (tracef @i{foo} '@i{foo})).\n"
+	    "@end lisp")
+#define FUNC_NAME s_scm_makmacro
+{
+  scm_c_issue_deprecation_warning
+    ("The function procedure->macro is deprecated, and so are"
+     " non-memoizing macros in general.  Use memoizing macros"
+     " or r5rs macros instead.");
+
+  SCM_VALIDATE_PROC (1, code);
+  return makmac (code, 1);
+}
+#undef FUNC_NAME
+
+#endif
+
+
 SCM_DEFINE (scm_macro_p, "macro?", 1, 0, 0, 
             (SCM obj),
 	    "Return @code{#t} if @var{obj} is a regular macro, a memoizing macro or a\n"
 	    "syntax transformer.")
 #define FUNC_NAME s_scm_macro_p
 {
-  return SCM_BOOL (SCM_TYP16_PREDICATE (scm_tc16_macro, obj));
+  return scm_from_bool (SCM_SMOB_PREDICATE (scm_tc16_macro, obj));
 }
 #undef FUNC_NAME
 
 
 SCM_SYMBOL (scm_sym_syntax, "syntax");
+#if SCM_ENABLE_DEPRECATED == 1
 SCM_SYMBOL (scm_sym_macro, "macro");
+#endif
 SCM_SYMBOL (scm_sym_mmacro, "macro!");
+SCM_SYMBOL (scm_sym_bimacro, "builtin-macro!");
 
 SCM_DEFINE (scm_macro_type, "macro-type", 1, 0, 0, 
             (SCM m),
@@ -173,13 +186,16 @@ SCM_DEFINE (scm_macro_type, "macro-type", 1, 0, 0,
 	    "returned.")
 #define FUNC_NAME s_scm_macro_type
 {
-  if (!SCM_TYP16_PREDICATE (scm_tc16_macro, m))
+  if (!SCM_SMOB_PREDICATE (scm_tc16_macro, m))
     return SCM_BOOL_F;
   switch (SCM_MACRO_TYPE (m))
     {
     case 0: return scm_sym_syntax;
+#if SCM_ENABLE_DEPRECATED == 1
     case 1: return scm_sym_macro;
+#endif
     case 2: return scm_sym_mmacro;
+    case 3: return scm_sym_bimacro;
     default: scm_wrong_type_arg (FUNC_NAME, 1, m);
     }
 }
@@ -191,7 +207,7 @@ SCM_DEFINE (scm_macro_name, "macro-name", 1, 0, 0,
 	    "Return the name of the macro @var{m}.")
 #define FUNC_NAME s_scm_macro_name
 {
-  SCM_VALIDATE_SMOB (1,m,macro);
+  SCM_VALIDATE_SMOB (1, m, macro);
   return scm_procedure_name (SCM_PACK (SCM_SMOB_DATA (m)));
 }
 #undef FUNC_NAME
@@ -202,7 +218,7 @@ SCM_DEFINE (scm_macro_transformer, "macro-transformer", 1, 0, 0,
 	    "Return the transformer of the macro @var{m}.")
 #define FUNC_NAME s_scm_macro_transformer
 {
-  SCM_VALIDATE_SMOB (1,m,macro);
+  SCM_VALIDATE_SMOB (1, m, macro);
   return ((SCM_CLOSUREP (SCM_PACK (SCM_SMOB_DATA (m)))) ?
 	  SCM_PACK(SCM_SMOB_DATA (m)) : SCM_BOOL_F);
 }
