@@ -54,6 +54,7 @@
 
 SCM_GLOBAL_SYMBOL (scm_sym_dot, ".");
 SCM_SYMBOL (scm_keyword_prefix, "prefix");
+SCM_SYMBOL (scm_keyword_postfix, "postfix");
 
 scm_t_option scm_read_opts[] = {
   { SCM_OPTION_BOOLEAN, "copy", 0,
@@ -63,7 +64,7 @@ scm_t_option scm_read_opts[] = {
   { SCM_OPTION_BOOLEAN, "case-insensitive", 0,
     "Convert symbols to lower case."},
   { SCM_OPTION_SCM, "keywords", SCM_UNPACK (SCM_BOOL_F),
-    "Style of keyword recognition: #f or 'prefix."}
+    "Style of keyword recognition: #f, 'prefix or 'postfix."}
 #if SCM_ENABLE_ELISP
   ,
   { SCM_OPTION_BOOLEAN, "elisp-vectors", 0,
@@ -181,6 +182,10 @@ static SCM *scm_read_hash_procedures;
   (((_chr) <= UCHAR_MAX) ? tolower (_chr) : (_chr))
 
 
+#ifndef HAVE_DECL_STRNCASECMP
+extern int strncasecmp (char const *s1, char const *s2, size_t n);
+#endif
+
 #ifndef HAVE_STRNCASECMP
 /* XXX: Use Gnulib's `strncasecmp ()'.  */
 
@@ -317,7 +322,7 @@ scm_read_sexp (int chr, SCM port)
   register int c;
   register SCM tmp;
   register SCM tl, ans = SCM_EOL;
-  SCM tl2 = SCM_EOL, ans2 = SCM_EOL, copy = SCM_BOOL_F;;
+  SCM tl2 = SCM_EOL, ans2 = SCM_EOL, copy = SCM_BOOL_F;
   static const int terminating_char = ')';
 
   /* Need to capture line and column numbers here. */
@@ -557,15 +562,19 @@ static SCM
 scm_read_mixed_case_symbol (int chr, SCM port)
 {
   SCM result, str = SCM_EOL;
-  int overflow = 0;
+  int overflow = 0, ends_with_colon = 0;
   char buffer[READER_BUFFER_SIZE];
   size_t read = 0;
+  int postfix = scm_is_eq (SCM_PACK (SCM_KEYWORD_STYLE), scm_keyword_postfix);
 
   scm_ungetc (chr, port);
   do
     {
       overflow = read_token (port, buffer, sizeof (buffer), &read);
 
+      if (read > 0)
+	ends_with_colon = (buffer[read - 1] == ':');
+
       if ((overflow) || (scm_is_pair (str)))
 	str = scm_cons (scm_from_locale_stringn (buffer, read), str);
     }
@@ -575,12 +584,21 @@ scm_read_mixed_case_symbol (int chr, SCM port)
     {
       str = scm_string_concatenate (scm_reverse_x (str, SCM_EOL));
       result = scm_string_to_symbol (str);
+
+      /* Per SRFI-88, `:' alone is an identifier, not a keyword.  */
+      if (postfix && ends_with_colon && (scm_c_string_length (result) > 1))
+	result = scm_symbol_to_keyword (result);
     }
   else
-    /* For symbols smaller than `sizeof (buffer)', we don't need to recur to
-       Scheme strings.  Therefore, we only create one Scheme object (a
-       symbol) per symbol read.  */
-    result = scm_from_locale_symboln (buffer, read);
+    {
+      /* For symbols smaller than `sizeof (buffer)', we don't need to recur
+	 to Scheme strings.  Therefore, we only create one Scheme object (a
+	 symbol) per symbol read.  */
+      if (postfix && ends_with_colon && (read > 1))
+	result = scm_from_locale_keywordn (buffer, read - 1);
+      else
+	result = scm_from_locale_symboln (buffer, read);
+    }
 
   return result;
 }
