@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2004 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2004, 2006, 2007 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -22,6 +22,7 @@
 #  include <config.h>
 #endif
 
+#include <fcntl.h>      /* for mingw */
 #include <signal.h>
 #include <stdio.h>
 #include <errno.h>
@@ -36,6 +37,14 @@
 #include "libguile/validate.h"
 #include "libguile/scmsigs.h"
 
+#ifdef HAVE_IO_H
+#include <io.h>  /* for mingw _pipe() */
+#endif
+
+#ifdef HAVE_PROCESS_H
+#include <process.h>    /* for mingw */
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -50,7 +59,7 @@
 /* This weird comma expression is because Sleep is void under Windows. */
 #define sleep(sec) (Sleep ((sec) * 1000), 0)
 #define usleep(usec) (Sleep ((usec) / 1000), 0)
-#define kill(pid, sig) raise (sig)
+#define pipe(fd) _pipe (fd, 256, O_BINARY)
 #endif
 
 
@@ -106,6 +115,12 @@ close_1 (SCM proc, SCM arg)
 }
 
 #if SCM_USE_PTHREAD_THREADS
+/* On mingw there's no notion of inter-process signals, only a raise()
+   within the process itself which apparently invokes the registered handler
+   immediately.  Not sure how well the following code will cope in this
+   case.  It builds but it may not offer quite the same scheme-level
+   semantics as on a proper system.  If you're relying on much in the way of
+   signal handling on mingw you probably lose anyway.  */
 
 static int signal_pipe[2];
 
@@ -120,23 +135,47 @@ take_signal (int signum)
 #endif
 }
 
+typedef struct {
+  ssize_t res;
+  int fd;
+  char *buf;
+  size_t n;
+} read_without_guile_data;
+
+static void *
+do_read_without_guile (void *raw_data)
+{
+  read_without_guile_data *data = (read_without_guile_data *)raw_data;
+  data->res = read (data->fd, data->buf, data->n);
+  return NULL;
+}
+
+static ssize_t
+read_without_guile (int fd, char *buf, size_t n)
+{
+  read_without_guile_data data;
+  data.fd = fd;
+  data.buf = buf;
+  data.n = n;
+  scm_without_guile (do_read_without_guile, &data);
+  return data.res;
+}
+
 static SCM
 signal_delivery_thread (void *data)
 {
-  sigset_t all_sigs;
-  scm_t_guile_ticket ticket;
   int n, sig;
   char sigbyte;
-
+#if HAVE_PTHREAD_SIGMASK  /* not on mingw, see notes above */
+  sigset_t all_sigs;
   sigfillset (&all_sigs);
   scm_i_pthread_sigmask (SIG_SETMASK, &all_sigs, NULL);
+#endif
 
   while (1)
     {
-      ticket = scm_leave_guile ();
-      n = read (signal_pipe[0], &sigbyte, 1);
+      n = read_without_guile (signal_pipe[0], &sigbyte, 1);
       sig = sigbyte;
-      scm_enter_guile (ticket);
       if (n == 1 && sig >= 0 && sig < NSIG)
 	{
 	  SCM h, t;
@@ -149,6 +188,8 @@ signal_delivery_thread (void *data)
       else if (n < 0 && errno != EINTR)
 	perror ("error in signal delivery thread");
     }
+
+  return SCM_UNSPECIFIED;	/* not reached */
 }
 
 static void
@@ -569,7 +610,9 @@ SCM_DEFINE (scm_sleep, "sleep", 1, 0, 0,
            (SCM i),
 	    "Wait for the given number of seconds (an integer) or until a signal\n"
 	    "arrives.  The return value is zero if the time elapses or the number\n"
-	    "of seconds remaining otherwise.")
+	    "of seconds remaining otherwise.\n"
+	    "\n"
+	    "See also @code{usleep}.")
 #define FUNC_NAME s_scm_sleep
 {
   return scm_from_uint (scm_std_sleep (scm_to_uint (i)));
@@ -578,7 +621,17 @@ SCM_DEFINE (scm_sleep, "sleep", 1, 0, 0,
 
 SCM_DEFINE (scm_usleep, "usleep", 1, 0, 0,
            (SCM i),
-	    "Sleep for @var{i} microseconds.")
+	    "Wait the given period @var{usecs} microseconds (an integer).\n"
+	    "If a signal arrives the wait stops and the return value is the\n"
+	    "time remaining, in microseconds.  If the period elapses with no\n"
+	    "signal the return is zero.\n"
+	    "\n"
+	    "On most systems the process scheduler is not microsecond accurate and\n"
+	    "the actual period slept by @code{usleep} may be rounded to a system\n"
+	    "clock tick boundary.  Traditionally such ticks were 10 milliseconds\n"
+	    "apart, and that interval is often still used.\n"
+	    "\n"
+	    "See also @code{sleep}.")
 #define FUNC_NAME s_scm_usleep
 {
   return scm_from_ulong (scm_std_usleep (scm_to_ulong (i)));
@@ -591,7 +644,7 @@ SCM_DEFINE (scm_raise, "raise", 1, 0, 0,
 	    "@var{sig} is as described for the kill procedure.")
 #define FUNC_NAME s_scm_raise
 {
-  if (kill (getpid (), scm_to_int (sig)) != 0)
+  if (raise (scm_to_int (sig)) != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
 }
