@@ -1,6 +1,6 @@
 ;;; srfi-19.scm --- Time/Date Library
 
-;; 	Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+;; 	Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007 Free Software Foundation, Inc.
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -14,7 +14,7 @@
 ;; 
 ;; You should have received a copy of the GNU Lesser General Public
 ;; License along with this library; if not, write to the Free Software
-;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 ;;; Author: Rob Browning <rlb@cs.utexas.edu>
 ;;;         Originally from SRFI reference implementation by Will Fitzgerald.
@@ -248,7 +248,8 @@
 ;; each entry is (tai seconds since epoch . # seconds to subtract for utc)
 ;; note they go higher to lower, and end in 1972.
 (define priv:leap-second-table
-  '((915148800 . 32)
+  '((1136073600 . 33)
+    (915148800 . 32)
     (867715200 . 31)
     (820454400 . 30)
     (773020800 . 29)
@@ -387,14 +388,6 @@
   (let ((run-time (get-internal-run-time)))
     (make-time
      time-process
-     (quotient run-time internal-time-units-per-second)
-     (* (remainder run-time internal-time-units-per-second)
-        priv:ns-per-guile-tick))))
-
-(define (priv:current-time-process)
-  (let ((run-time (get-internal-run-time)))
-    (list
-     'time-process
      (* (remainder run-time internal-time-units-per-second)
         priv:ns-per-guile-tick)
      (quotient run-time internal-time-units-per-second))))
@@ -856,10 +849,12 @@
         (hour (date-hour date))
         (day (date-day date))
         (month (date-month date))
-        (year (date-year date)))
+        (year (date-year date))
+        (offset (date-zone-offset date)))
     (+ (priv:encode-julian-day-number day month year)
        (- 1/2)
-       (+ (/ (+ (* hour 60 60)
+       (+ (/ (+ (- offset)
+                (* hour 60 60)
                 (* minute 60)
                 second
                 (/ nanosecond priv:nano))
