@@ -1,4 +1,4 @@
-/* Copyright (C) 1999,2000,2001,2003,2004, 2006, 2007 Free Software Foundation, Inc.
+/* Copyright (C) 1999,2000,2001,2003,2004, 2006, 2007, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,14 +15,16 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "libguile.h"
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <libguile.h>
 
 #include <stdio.h>
 #include <assert.h>
 #include <string.h>
 
-#include "config.h"
-
 #ifdef HAVE_INTTYPES_H
 # include <inttypes.h>
 #elif (!defined PRIiMAX)
